/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package client;

import data.*;
import java.io.BufferedReader;
import java.io.Console;
import java.io.InputStreamReader;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Persistence;
import webservice.Exception_Exception;

public class Client
{
    private static InputStreamReader isr;
    private static BufferedReader br;
    private static EntityManager em;
    private static int devid;
    
    public static void main(String[] args) throws Exception
    {
        isr = new InputStreamReader(System.in);
        br = new BufferedReader(isr);
        em = Persistence.createEntityManagerFactory("clientPU").createEntityManager();
        devid = 22;
        
        
        while (true)
        {
            switch (ShowAndGetMenuOption())
            {
                case 1:
                    AddTask();
                    break;
                case 2:
                    EditTask();
                    break;
                case 3:
                    DeleteTask();
                    break;
                case 4:
                    Sync();
                    break;
                case 5:
                    ListTasks();
                    break;
                case 6:
                    return;
            }
        }
    }
    
    static void AuditChange(int mode, int taskid)
    {
        ClientAudit audit = new ClientAudit();
        audit.setId(0);
        audit.setMode(mode);
        audit.setTaskid(taskid);
        
        em.getTransaction().begin();
        em.persist(audit);
        em.getTransaction().commit();
    }
    
    static int ShowAndGetMenuOption() throws Exception
    {
        System.out.print("1 - add, 2 - edit, 3 - delete, 4 - sync, 5 - list, 6 - quit\nEnter: ");
        return Integer.parseInt(br.readLine());
    }
    
    static void AddTask() throws Exception
    {
        System.out.print("Enter task name: ");
        
        String name = br.readLine();
        
        ClientTask task = new ClientTask();
        task.setId(0);
        task.setSvrid(0);
        task.setName(name);
        
        em.getTransaction().begin();
        em.persist(task);
        em.getTransaction().commit();
        
        AuditChange(1, task.getId());
    }
    
    static void EditTask() throws Exception
    {
        ListTasks();
        System.out.print("Edit task id: ");
        int id = Integer.parseInt(br.readLine());
        
        em.getTransaction().begin();
        
        ClientTask tsk = (ClientTask) em.createNamedQuery("ClientTask.findById").setParameter("id", id).getSingleResult();
        
        System.out.print("Task name: ");
        tsk.setName(br.readLine());
        
        em.getTransaction().commit();
        
        AuditChange(2, id);
    }
    
    static void DeleteTask() throws Exception
    {
        ListTasks();
        System.out.print("Edit task id: ");
        int id = Integer.parseInt(br.readLine());
        int svrid;
        
        em.getTransaction().begin();
        
        ClientTask tsk = (ClientTask) em.createNamedQuery("ClientTask.findById").setParameter("id", id).getSingleResult();
        svrid = tsk.getSvrid();
        
        em.remove(tsk);
        
        em.getTransaction().commit();
        
        AuditChange(3, svrid);
    }
    
    static void DeleteAudit(Object audit)
    {
        em.getTransaction().begin();
        em.remove(audit);
        em.getTransaction().commit();
    }
    
    static void ListTasks()
    {
        List<ClientTask> tasks = em.createNamedQuery("ClientTask.findAll").getResultList();
        
        for (ClientTask task : tasks)
        {
            System.out.printf("[%d] %s [%s]\n", task.getId(), task.getName(), task.getSvrid() == 0 ? "Unsynced" : "Synced" );
        }
    }
    
    static void UpdateLastSynced(Integer lastId)
    {
        em.getTransaction().begin();
        ClientInfo cinfo = (ClientInfo) em.createNamedQuery("ClientInfo.findByName").setParameter("name", "lastupdate").getSingleResult();
        cinfo.setValue(lastId.toString());
        em.getTransaction().commit();
    }
    
    static void Sync() throws Exception
    {
        //
        //First take our updates and throw them on the server
        //
        List<ClientAudit> ownUpdates = em.createNamedQuery("ClientAudit.findAll").getResultList();
        
        for (ClientAudit audit : ownUpdates)
        {
            ClientTask ownTask = null;
            
            //try to grab a local version of the task
            try
            {
                //if add or edit
                if (audit.getMode() < 3)
                {
                    ownTask = (ClientTask) em.createNamedQuery("ClientTask.findById").setParameter("id", audit.getTaskid()).getSingleResult();
                
                    if (ownTask == null)
                        throw new Exception("");
                }
            }
            catch (Exception e)
            {
                System.out.println("[sync/warn] Task was deleted on client before synced to server: " + audit.getTaskid());
                DeleteAudit(audit);
                continue; //move on to the next one
            }
            
            //add
            if (audit.getMode() == 1)
            {
                int serverid = addTask(ownTask.getName(), devid);
                
                //update local to remote server id
                em.getTransaction().begin();
                ownTask.setSvrid(serverid);
                em.getTransaction().commit();
                
                System.out.printf("[sync/info/add] Synced local to remote (local:remote) (%d:%d)\n", ownTask.getId(), ownTask.getSvrid());
            }
            else if (audit.getMode() == 2)
            {
                editTask(ownTask.getName(), ownTask.getSvrid(), devid);
                
                System.out.printf("[sync/info/edit] Synced local to remote (local:remote) (%d:%d)\n", ownTask.getId(), ownTask.getSvrid());
            }
            else if (audit.getMode() == 3)
            {
                deleteTask(audit.getTaskid(), devid);
                System.out.printf("[sync/info/delete] Synced local to remote\n");
            }
            
            //delete audit to prevent another update
            DeleteAudit(audit);
        } // end for
        
        
        //
        //Now grab all the updates from the server
        //
        ClientInfo inf = (ClientInfo) em.createNamedQuery("ClientInfo.findByName").setParameter("name", "lastupdate").getSingleResult();
        int lastUpdate = Integer.parseInt(inf.getValue());
        
        List<webservice.ServerAudit> remoteAudits = getAudits(devid, lastUpdate);
        
        for (webservice.ServerAudit audit : remoteAudits)
        {
            webservice.ServerTask remTask = null;
            
            try
            {
                if (audit.getMode() < 3)
                {
                    remTask = getTask(audit.getTaskid());
                    
                    if (remTask == null)
                        throw new Exception("");
                }
            }
            catch (Exception e)
            {
                UpdateLastSynced(audit.getId());
                System.out.println("[sync/warn] Task was deleted on client before synced to server " + audit.getTaskid());
                continue;
            }
            
            em.getTransaction().begin();
            
            //add a task from the server
            if (audit.getMode() == 1)
            {
                ClientTask task = new ClientTask();
                task.setId(0);
                task.setSvrid(remTask.getId());
                task.setName(remTask.getName());
                em.persist(task);
                
                System.out.printf("[sync/info/get/add] Synced remote to local (local:remote) (%d:%d)\n", task.getId(), task.getSvrid());
            }
            else if (audit.getMode() == 2)
            {
                ClientTask task = null;
                
                try
                {
                    task = (ClientTask) em.createNamedQuery("ClientTask.findBySvrid").setParameter("svrid", remTask.getId()).getSingleResult();
                    
                    if (task == null)
                        throw new Exception("");
                }
                catch (Exception e)
                {
                    System.out.printf("[sync/info/warn] Task was deleted on server before synced to client\n");
                    em.getTransaction().rollback();
                    UpdateLastSynced(audit.getId());
                    continue;
                }
                
                task.setName(remTask.getName());
                
                System.out.printf("[sync/info/get/edit] Synced remote to local (local:remote) (%d:%d)\n", task.getId(), task.getSvrid());
            }
            else if (audit.getMode() == 3)
            {
                ClientTask task = null;
                
                try
                {
                    task = (ClientTask) em.createNamedQuery("ClientTask.findBySvrid").setParameter("svrid", remTask.getId()).getSingleResult();
                    
                    if (task == null)
                        throw new Exception("");
                }
                catch (Exception e)
                {
                    System.out.printf("[sync/info/warn] Task was deleted on server before synced to client\n");
                    em.getTransaction().rollback();
                    UpdateLastSynced(audit.getId());
                    continue;
                }
                
                em.remove(task);
                
                System.out.printf("[sync/info/get/delete] Synced remote to local\n");
            }
            
            em.getTransaction().commit();
            UpdateLastSynced(audit.getId());
        }
    }

    private static int addTask(java.lang.String name, int devid) throws Exception_Exception
    {
        webservice.Sync_Service service = new webservice.Sync_Service();
        webservice.Sync port = service.getSyncPort();
        return port.addTask(name, devid);
    }

    private static boolean deleteTask(int taskid, int devid) throws Exception_Exception
    {
        webservice.Sync_Service service = new webservice.Sync_Service();
        webservice.Sync port = service.getSyncPort();
        return port.deleteTask(taskid, devid);
    }

    private static boolean editTask(java.lang.String name, int taskid, int devid) throws Exception_Exception
    {
        webservice.Sync_Service service = new webservice.Sync_Service();
        webservice.Sync port = service.getSyncPort();
        return port.editTask(name, taskid, devid);
    }

    private static java.util.List<webservice.ServerAudit> getAudits(int devid, int lastupdate)
    {
        webservice.Sync_Service service = new webservice.Sync_Service();
        webservice.Sync port = service.getSyncPort();
        return port.getAudits(devid, lastupdate);
    }

    private static webservice.ServerTask getTask(int taskid)
    {
        webservice.Sync_Service service = new webservice.Sync_Service();
        webservice.Sync port = service.getSyncPort();
        return port.getTask(taskid);
    }
}