/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package webservice;

import data.*;
import javax.annotation.Resource;
import javax.jws.WebService;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.transaction.UserTransaction;

/**
 *
 * @author Matt
 */
@WebService(serviceName = "Sync")
public class Sync
{
    //inject the persistence unit factory
    @PersistenceUnit private EntityManagerFactory emf;
    
    //inject the transaction for the persistence unit
    @Resource UserTransaction ut;
    
    @WebMethod(operationName = "AddTask")
    public int AddTask(@WebParam(name = "name") String name, @WebParam(name = "devid") int devid) throws Exception
    {
        try
        {
            ServerTask tsk = new ServerTask();
            tsk.setName(name);
            tsk.setId(0);

            Persist(tsk);

            AuditItem(tsk.getId(), 1, devid);

            return tsk.getId();
        }
        catch (Exception e)
        {
            return -1;
        }
    }
    
    private void AuditItem(int taskid, int mode, int devid) throws Exception
    {
        ServerAudit aud = new ServerAudit();
        aud.setId(0);
        aud.setMode(mode);
        aud.setTaskid(taskid);
        aud.setDevid(devid);
        
        Persist(aud);
    }
    
    private void Persist(Object o) throws Exception
    {
        ut.begin();
        emf.createEntityManager().persist(o);
        ut.commit();
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "EditTask")
    public boolean EditTask(@WebParam(name = "name") String name, @WebParam(name = "taskid") int taskid, @WebParam(name = "devid") int devid) throws Exception
    {
        try
        {
            ut.begin();

            ServerTask tsk = (ServerTask) emf.createEntityManager().createNamedQuery("ServerTask.findById").setParameter("id", taskid).getSingleResult();

            if (tsk == null)
                return false;

            tsk.setName(name);

            ut.commit();

            AuditItem(tsk.getId(), 2, devid);

            return true;
        }
        catch (Exception e)
        {
            return false;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "DeleteTask")
    public boolean DeleteTask(@WebParam(name = "taskid") int taskid, @WebParam(name = "devid") int devid) throws Exception
    {
        EntityManager em = emf.createEntityManager();
        
        try
        {
            ut.begin();

            ServerTask tsk = (ServerTask) em.createNamedQuery("ServerTask.findById").setParameter("id", taskid).getSingleResult();

            em.remove(tsk);

            ut.commit();

            AuditItem(taskid, 3, devid);

            return true;
        }
        catch (Exception e)
        {
            return false;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "GetTask")
    public ServerTask GetTask(@WebParam(name = "taskid") int taskid)
    {
        try
        {
            return (ServerTask) emf.createEntityManager().createNamedQuery("ServerTask.findById").setParameter("id", taskid).getSingleResult();
        }
        catch (Exception e)
        {
            return null;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "GetAudits")
    public java.util.List<ServerAudit> GetAudits(@WebParam(name = "devid") int devid, @WebParam(name = "lastupdate") int lastupdate)
    {
        try
        {
            return emf.createEntityManager().createNamedQuery("ServerAudit.findWhere").setParameter("lastupdate", lastupdate).setParameter("devid", devid).getResultList();
        }
        catch (Exception e)
        {
            return null;
        }
    }
}
